'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _exception = require('./utils/exception');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (config) {
  // define default configuration options
  var defaultConfig = {
    test: /\/api\/(v[0-9]+).*/,
    entryPoint: 'app.js',
    apiPath: '',
    instance: null
  };

  // merge default configuration options with user defined config options
  var mergedConfig = Object.assign({}, defaultConfig, config);

  // get config options from merged configs
  var test = mergedConfig.test,
      entryPoint = mergedConfig.entryPoint,
      apiPath = mergedConfig.apiPath,
      instance = mergedConfig.instance;


  if (!apiPath) {
    // throw exception if the apiPath is not specified
    throw new _exception.Exception('You must explicitly specify a path to where the APIs reside');
  }

  if (!instance) {
    // throw an exception if the express instance is undefined
    throw new _exception.Exception('You must explicitly set an instance of express');
  }

  if (typeof instance !== 'function') {
    // throw an exception if the instance is not a function
    throw new _exception.Exception('An instance of express must be a function but got type ' + (typeof instance === 'undefined' ? 'undefined' : _typeof(instance)));
  }

  return function (httpRequest, httpResponse, next) {
    // test if the version number/type exist in the url
    var testUrl = httpRequest.url.match(test);
    var version = testUrl ? testUrl[1] : '';

    if (version) {
      // if we have the version number/type in url
      // normalize path to the entry point
      var fullPath = _path2.default.normalize(apiPath + '/' + version + '/' + entryPoint);
      if (_fs2.default.existsSync(fullPath)) {
        // check if the entry point exist
        if (typeof require(fullPath).default === 'function') {
          require(fullPath).default(instance); // import the entry point
        } else {
          require(fullPath)(instance);
        }
        next();
      }
    }
    throw new _exception.Exception('Not Found');
  };
};